      SUBROUTINE MKAdler(cosThAd, phiAd)
C     Boost particles to get the Adler angles needed as input for
C     Minoo's cross-section calculations. Needed for the reweight
C     routine
C     Use the common block data to calculate the Adler angles
      implicit none

C     Get the filled common blocks
#include "nework.h"
#include "vcwork.h"
#include "posinnuc.h"
#include "rscons.h"

C     The returns
      real cosThAd, phiAd
C     Double versions for internal processing
      double precision cosThAdPr, phiAdPr

C     Nucleon and pion 4-momentum vectors
      double precision Nucl(4),Pion(4),Res(4),ResNeg(4)
C     Pion in resonance frame
      double precision PiOut(4)

C     Outgoing lepton and neutrino 4-momentum vectors
C     N.B. "Out" refers to the 4-momentum in the resonance frame
      double precision Lep(4),Neu(4)

C     The returns
      double precision LepOut(4),NeuOut(4)
C     The 3-vectors of neutrino and lepton
      double precision NeuVect(3),LepVect(3)
C     Looper
      integer I
C     The x axis, y axis and z axis
      double precision xAxis(3),yAxis(3),zAxis(3)
C     Auxillary planes
      double precision PiOnZ(3),PiPlane(3)

C     4 particle is nucleon, 5th particle is pion ALWAYS
      DO 10 I=1,3
        Neu(I) = dble(PNE(I, 1))
        Lep(I) = dble(PNE(I, 3))
        Nucl(I) = dble(PNE(I, 4))
        Pion(I) = dble(PNE(I, 5))
        Res(I) = Nucl(I) + Pion(I)
        ResNeg(I) = -1.0*Res(I)
  10  CONTINUE

      Neu(4) = sqrt(Neu(1)**2+Neu(2)**2+Neu(3)**2)

C     Mass of nucleon
      IF (IPNE(4) .EQ. 2112) THEN
        Nucl(4) = dble(XMNE)
      ELSE
        Nucl(4) = dble(XMP)
      ENDIF
C     Set fourth component to be energy
      Nucl(4) = sqrt(Nucl(1)**2+Nucl(2)**2+
     &               Nucl(3)**2+Nucl(4)**2)

C     Mass of pion (charged)
      IF (ABS(IPNE(5)).EQ.211) THEN
        Pion(4) = dble(0.13957)
C     Mass of pion (neutral)
      ELSE
        Pion(4) = dble(0.13497)
      ENDIF
      Pion(4) = sqrt(Pion(1)**2+Pion(2)**2+
     &               Pion(3)**2+Pion(4)**2)

C     Set lepton mass for lepton energy by looking at PID
      IF (abs(IPNE(3)).EQ.11) THEN
         Lep(4) = dble(XME)
      ELSE IF (abs(IPNE(3)).EQ.13) THEN
         Lep(4) = dble(XMMU)
      ELSE IF (abs(IPNE(3)).EQ.15) THEN
         Lep(4) = dble(XMTAU)
      ELSE 
         Lep(4) = dble(0.)
      endIF

C     Outgoing lepton energy
      Lep(4) = sqrt(Lep(1)**2+Lep(2)**2+
     &              Lep(3)**2+Lep(4)**2)

      Res(4) = Pion(4)+Nucl(4)
      ResNeg(4) = Res(4)

C     Do Lorentz boost using CERNLIB
C     First argument is 4-vector (P,E), second argument is 4-vector
C     (p,e), third argument is return 4-vector (p',e') --- i.e. boost
C     second argument to first argument frame

C     Boost the neutrino
      call LOREN4_DBL(RES, Neu, NeuOut)

C     Boost the pion
      call LOREN4_DBL(RES, Pion, PiOut)

C     Boost the muon
      call LOREN4_DBL(RES, Lep, LepOut)

C     Now we have the pion, outgoing lepton and incoming neutrino boosted in to the resonance rest frame
C     Make the zaxis defined as the momentum transfer in the resonance
C     rest frame
      DO 30 I=1,3
        zAxis(I) = (NeuOut(I) - LepOut(I))
  30  CONTINUE

C     Normalise the z-axis vector to unit
      call normalise(zAxis)

C     Cos theta Adler is THEN defined as the angle between the pion in
C     the resonant rest frame and the z-axis
C     So we need the angle between PiOut and zAxis
C     zaxis is already unit normalised so only 
      cosThAdPr= (zAxis(1)*PiOut(1)+zAxis(2)*PiOut(2)+zAxis(3)*PiOut(3)
     &)/sqrt(PiOut(1)**2+PiOut(2)**2+PiOut(3)**2)

C     Need to reduce the 4-vector to a three vector to use CERNLIB cross
C     product
      DO 50 I=1,3
        NeuVect(I) = NeuOut(I)
        LepVect(I) = LepOut(I)
  50  CONTINUE

C     Now to get phi, eurgh
C     Already have zaxis defined above, now define y and x-axis
      call CROSS_DBL(NeuVect, LepVect, yAxis)
      call normalise(yAxis)

C     Then finally make the x-axis as y X z
      call CROSS_DBL(yAxis, zAxis, xAxis)
      call normalise(xAxis)

C     Now have a normalised x,y,z coordinate system
C     Get the projection of the pion onto the z-axis and make the plane
      DO 60 I=1,3
        PiOnZ(I)=zAxis(I)*(zAxis(1)*PiOut(1)+
     &                     zAxis(2)*PiOut(2)+
     &                     zAxis(3)*PiOut(3))
        PiPlane(I)=PiOut(I)-PiOnZ(I)
  60  CONTINUE

C     Calculate the phi Adler angle
      phiAdPr=acos((PiPlane(1)*xAxis(1)+
     &              PiPlane(2)*xAxis(2)+
     &              PiPlane(3)*xAxis(3))/
     &      sqrt(PiPlane(1)**2+PiPlane(2)**2+PiPlane(3)**2))

C     Check rotation and put in 
      IF (PiPlane(2).LT.0) THEN
        phiAdPr=2.0*pi-phiAdPr
      ENDIF

C     Finally convert to single precision
      phiAd=phiAdPr
      cosThAd=cosThAdPr

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE NORMALISE(INPUT)
C     helper function to normalise a three vector, e.g. momentum
      implicit none
      double precision input(3)
      integer i

      DO 10 I=1,3
        input(I) = input(I)/sqrt(input(1)**2+
     &             input(2)**2+input(3)**2)
  10  CONTINUE

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE CROSS_DBL(X,Y,Z)
C     helper function to cross-product in double precision
C     Just a copy of CERNLIB CROSS function but better precision
      implicit none
      double precision X(3), Y(3), Z(3)
      double precision Z1, Z2

      Z1=X(2)*Y(3)-X(3)*Y(2)
      Z2=X(3)*Y(1)-X(1)*Y(3)
      Z(3)=X(1)*Y(2)-X(2)*Y(1)
      Z(1)=Z1
      Z(2)=Z2

      RETURN
      END

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE LOREN4_DBL(DIR,P4IN,P4OUT)
C     helper function to Lorentz boost in double precision
C     Just a copy of CERNLIB LOREN4function but better precision
      IMPLICIT NONE
      DOUBLE PRECISION PCM2, ONMCM, EPBETA, PROD
      DOUBLE PRECISION DIR(4),P4IN(4),P4OUT(4)
      integer i

      PCM2=DIR(1)*DIR(1)+DIR(2)*DIR(2)+DIR(3)*DIR(3)
      ONMCM=1.D0/ SQRT (DIR(4)*DIR(4)-PCM2)

      EPBETA=P4IN(1)*DIR(1)+P4IN(2)*DIR(2)+P4IN(3)*DIR(3)
      PROD=EPBETA*(DIR(4)*ONMCM-1.D0)/PCM2-P4IN(4)*ONMCM
      P4OUT(4)=ONMCM*(P4IN(4)*DIR(4)-EPBETA)
      DO 50 I=1,3
        P4OUT(I)=P4IN(I)+DIR(I)*PROD
  50  CONTINUE

      RETURN
      END
