***********************************************************************
*     ---------------------------------------------------
      SUBROUTINE MKDCRS(IMODE, E, Q2, W, COSTHETA, PHI, XMLEP, DIFCRS)
*     ---------------------------------------------------
*     ( purpose )
*       calculate Differential Cross-section d^2(sig)/dwdQ2d(theta)d(phi) using Minoo Kabirnezhad's single pion model.
*
*       Code largely inspired by rsdcrs.F to make easy to follow
*       Calls different C functions for the different interaction modes
*
*     ( input )
*       imode : Interaction mode
*               imode > 10  mans anti-neutrino
*       e     : Incident neutrino energy in lab. frame.
*               (Lab. frame is nucleon rest frame.)
*       q2    : sqaured momentum transfer.(minus sign)
*       w     : Rest mass of products(pion and nucreon).
*       costheta: cos theta as defined in the Adler frame
*       phi   : as defined in the Adler frame
*       xmlep : mass of lepton
*
*     ( output )
*       difcrs: differential cross-section.
*               If error occurs in this subroutine,difcrs
*               returns -9999.999
*
*     ( creation date and author )
*       2016 Oct: Clarence Wret and Minoo Kabirnezhad
*                 Final implementation in about May 2017
*
***********************************************************************

      IMPLICIT NONE

C     Define the inputs
      INTEGER IMODE
      REAL E, Q2, W, COSTHETA, PHI
      REAL diff_xsec, difcrs

C     Use the neutmodel and Rein Sehgal constants
#include "rscons.h"
#include "neutmodel.h"

      REAL XMLEP

C     Checkers for valid kinematic
      REAL XNU 
      Real*4 El, Plep, W2, X
      REAL K_0, Q_0

C-------------------------------------------------------------------------

C     Background resonance mass. Hardcode for now, make reweightable
C     option
      diff_xsec = 0.

      IF ( (IMODE.GT.17) .OR. ((IMODE.GT.7) .AND. (IMODE.LT.11) )) THEN
        write(*,*) "Wrong mode for Minoo model"
        stop
      ENDIF

C     Check the W range
      IF (W .LE. XMPI+XMN) THEN
        W = XMPI+XMN+1.E-5
      ENDIF

      W2 = W**2

C     KINEMATICAL QUANTITIES
C     Check that the Q2 is allowed
      IF ( ((2 * XMN * E) * (2 * XMN * E + XMN2 - W2) /
     $     (2 * XMN * E + XMN2) + Q2) .LT. -1.0E-5) THEN
        write(*,*) "Q2 range"
        GOTO 999
      ENDIF

      XNU = (W2 - XMN2 - Q2) / (2 * XMN)
      X = -Q2 / (2 * XMN * XNU)
      IF(XNU.GE.E) THEN
        write(*,*) "XNU > E" 
        GOTO 999
      ENDIF

      Q_0 = (W2 - XMN2 + XMPI2)/(2.*W)
      if (Q_0*Q_0 - XMPI2 .LT. 0) THEN
        write(*,*) "Q_02 < mpi2"
        GOTO 999
      endif

      K_0 = (W2 - XMN2 + Q2)/(2.*W)
      if (K_0*K_0 - Q2 .LT. 0) then
        write(*,*) "K_02 < Q2"
        GOTO 999
      endif

C     SELECT Channels for neutrino and anti-neutrino
C     Neutrino
      IF(IMODE.EQ.1)  GOTO 1001  ! CC 'P - PI+'
      IF(IMODE.EQ.2)  GOTO 1002  ! CC 'P - PI0'
      IF(IMODE.EQ.3)  GOTO 1003  ! CC 'N - PI+'
      IF(IMODE.EQ.4)  GOTO 1004  ! NC 'P - PI0'
      IF(IMODE.EQ.5)  GOTO 1005  ! NC 'N - PI+'
      IF(IMODE.EQ.6)  GOTO 1006  ! NC 'N - PI0'
      IF(IMODE.EQ.7)  GOTO 1007  ! NC 'P - PI-'

C     anti-Neutrino
      IF(IMODE.EQ.11)  GOTO 1011 ! CC 'N - PI-'
      IF(IMODE.EQ.12)  GOTO 1012 ! CC 'N - PI0'
      IF(IMODE.EQ.13)  GOTO 1013 ! CC 'P - PI-' 
      IF(IMODE.EQ.14)  GOTO 1014 ! NC 'P - PI0'
      IF(IMODE.EQ.15)  GOTO 1015 ! NC 'N - PI+' 
      IF(IMODE.EQ.16)  GOTO 1016 ! NC 'N - PI0'
      IF(IMODE.EQ.17)  GOTO 1017 ! NC 'P - PI-' 

C     Do a check on the W
      IF (W .LT. XMN+XMPI+1E-4) THEN
        W = XMN+XMPI+1E-4
      ENDIF

 1001 CONTINUE
      CALL Xsec_IM1(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

 1002 CONTINUE
      CALL Xsec_IM2(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

 1003 CONTINUE
      CALL Xsec_IM3(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

C     NC interactions don't need lepton mass passed
 1004 CONTINUE
      CALL Xsec_IM4(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1005 CONTINUE
      CALL Xsec_IM5(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1006 CONTINUE
      CALL Xsec_IM6(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1007 CONTINUE
      CALL Xsec_IM7(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1011 CONTINUE
      CALL Xsec_IM11(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

 1012 CONTINUE
      CALL Xsec_IM12(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

 1013 CONTINUE
      CALL Xsec_IM13(diff_xsec, E, -Q2, W, COSTHETA, PHI, XMLEP)
      GOTO 69

C     NC interactions don't need lepton mass passed
 1014 CONTINUE
      CALL Xsec_IM14(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1015 CONTINUE
      CALL Xsec_IM15(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1016 CONTINUE
      CALL Xsec_IM16(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 1017 CONTINUE
      CALL Xsec_IM17(diff_xsec, E, -Q2, W, COSTHETA)
      GOTO 69

 999  CONTINUE
      write (*,*) "Q2 IS WEIRD IN KABIRNEZHAD 1PI MODEL"
      write (*,*) "CALC = ",(2 * XMN * E) * (2 * XMN * E + XMN2 - W2) /
     $     (2 * XMN * E + XMN2)
      write (*,*) "-Q2 = ",-Q2
      diff_xsec = 0
      GOTO 69

C     Go here when we've calculated a cross-section
 69   CONTINUE
      difcrs = diff_xsec

      IF (ISNAN(difcrs)) THEN
        DIFCRS = 0
        WRITE(*,*) "FOUND NAN FOR CROSS-SECTION IN KABIRNEZHAD 1PI:"
        WRITE(*,*) "SETTING TO ZERO"
        WRITE(*,*) "MODE = ", IMODE
        WRITE(*,*) "E = ", E
        WRITE(*,*) "Q2 = ", Q2
        WRITE(*,*) "W = ", W
        WRITE(*,*) "COSTHETA = ", COSTHETA
        WRITE(*,*) "PHI = ", PHI
      ENDIF

      END
