//____________________________________________________________________________
/*
 Copyright (c) 2003-2010, GENIE Neutrino MC Generator Collaboration
 For the full text of the license visit http://copyright.genie-mc.org
 or see $GENIE/LICENSE

 Authors: Jim Dobson <J.Dobson07 \at imperial.ac.uk>
          Imperial College London

          Costas Andreopoulos <costas.andreopoulos \at stfc.ac.uk>
          STFC, Rutherford Appleton Laboratory 

	  Patrick de Perio <pdeperio \at physics.utoronto.ca>
	  University of Toronto

 For the class documentation see the corresponding header file.

 Important revisions after version 2.0.0 :
 @ Aug 01, 2009 - CA
   Was adapted from Jim's and Costas' T2K-specific GENIE reweighting code.
   First included in v2.5.1.
 @ May 18, 2010 - CA
   AdoptWghtCalc(std::string,NReWeightI*) allows user to decide which weight 
   calculator to include. Weight calculators are owned by NReWeight and are 
   identified by a name. Weight calculators can be retrieved via the 
   WghtCalc(std::string) method and their reweighting options can be fine-tuned.
 @ Apr 6, 2011 - PD
   Implemented in NEUT
*/
//____________________________________________________________________________
#include "NReWeight.h"

//#define _NEUT_REWEIGHT_DEBUG_

using namespace neut;
using namespace neut::rew;

// Have necard to set the defaults from NEUT
extern "C" {
  void necard_();
}

//____________________________________________________________________________
NReWeight::NReWeight()
{

}
//____________________________________________________________________________
NReWeight::~NReWeight()
{
  this->CleanUp();
}
//____________________________________________________________________________
void NReWeight::AdoptWghtCalc(std::string name, NReWeightI* wcalc)
{
  if(!wcalc) return;

  fWghtCalc.insert(std::map<std::string, NReWeightI*>::value_type(name,wcalc));
}
//____________________________________________________________________________
NReWeightI* NReWeight::WghtCalc(std::string name)
{ 
  std::map<std::string, NReWeightI*>::iterator iter = fWghtCalc.find(name);
  if(iter != fWghtCalc.end()) return iter->second;
  
  return 0;
}
//____________________________________________________________________________
NSystSet & NReWeight::Systematics(void)
{ 
  return fSystSet; 
}
//____________________________________________________________________________
void NReWeight::Reconfigure(void)
{
#ifdef _NEUT_REWEIGHT_DEBUG_
  std::cout << "NReWeight: Reconfiguring ...";
#endif 

  vector<neut::rew::NSyst_t> svec = fSystSet.AllIncluded();

  std::map<std::string, NReWeightI *>::iterator it = fWghtCalc.begin();
  for(; it != fWghtCalc.end(); ++it) {

      NReWeightI * wcalc = it->second;

      vector<neut::rew::NSyst_t>::const_iterator parm_iter = svec.begin();
      for(; parm_iter != svec.end(); ++parm_iter) {
          NSyst_t syst = *parm_iter;
          double val = fSystSet.Info(syst)->CurValue;
          wcalc->SetSystematic(syst, val);
      }//params

      wcalc->Reconfigure();

  }//weight calculators

#ifdef _NEUT_REWEIGHT_DEBUG_
  std::cout << "Done reconfiguring" << std::endl;
#endif
}
//____________________________________________________________________________
double NReWeight::CalcWeight() 
{
// Calculate weight for all tweaked physics parameters
  double weight = 1.0;
  fWeightXsec = 1.0;

  std::map<std::string, NReWeightI *>::iterator it = fWghtCalc.begin();
  for (; it != fWghtCalc.end(); ++it) {

#ifdef _NEUT_REWEIGHT_DEBUG_
    std::cout << "Before CalcWeight: " << std::endl;
    PrintNeutAll();
#endif

    NReWeightI * wcalc = it->second;
    double w = wcalc->CalcWeight();

#ifdef _NEUT_REWEIGHT_DEBUG_
    std::cout << "After CalcWeight: " << std::endl;
    PrintNeutAll();
#endif

#ifdef _NEUT_REWEIGHT_DEBUG_
    std::cout << "Calculator: " << it->first << " => wght = " << w << std::endl;	
#endif

    // Store weight from neutrino cross section variations only, 
    // not FSI (casc) or nuclear effects (e.g. piless decay).
    // This assumes the std::mapping is named with "xsec" as in 
    // ${T2KREWEIGHT}/src/T2KNeutReWeight.cxx.
    // This is used by ${NIWG}/neut/ana/NInputXsec for reweighting
    // the input (bare) cross sections.
    if ( (it->first).find("xsec") != std::string::npos ) {
      fWeightXsec *=w;
    }

    weight *= w;
  }

  return weight;
}

//____________________________________________________________________________
double NReWeight::CalcChisq(void) 
{
// Calculate the sum of penalty terms for all tweaked physics parameters
  double tot_chisq = 0.0;

  std::map<std::string, NReWeightI *>::iterator it = fWghtCalc.begin();
  for(; it != fWghtCalc.end(); ++it) {
    NReWeightI * wcalc = it->second;
    double chisq = wcalc->CalcChisq(); 
#ifdef _NEUT_REWEIGHT_DEBUG_
    //std::cout << "Calculator: " << it->first << " => chisq = " << chisq;	
#endif
    tot_chisq *= chisq;
  }
  return tot_chisq;
}
//____________________________________________________________________________
void NReWeight::CleanUp(void)
{
  std::map<std::string, NReWeightI *>::iterator it = fWghtCalc.begin();
  for(; it != fWghtCalc.end(); ++it) {
    NReWeightI * rw = it->second;
    if(rw) {
      delete rw;
      rw=0;
    }
  }
  fWghtCalc.clear();
}
//____________________________________________________________________________
void NReWeight::Print()
{
  vector<neut::rew::NSyst_t> syst_vec = this->Systematics().AllIncluded();
  int vec_size = syst_vec.size();

  std::cout << "NReWeight: Current set of systematic params:" << std::endl;	
  for(int i = 0; i < vec_size; i ++){
     std::cout << " --o "  << NSyst::AsString(syst_vec[i]) << " is set at " << this->Systematics().Info(syst_vec[i])->CurValue << std::endl;
  }		       	        

  double chi2val = this->CalcChisq();

  std::cout << "Chisq_{penalty} = " << chi2val << std::endl;
}
//____________________________________________________________________________
// Fill the NEUT nework common block for each event
void NReWeight::FillNeutEvent(NeutVect *nvect) {

  // neutcore/nework.h has documentation on nework common block
  // Interaction mode
  nework_.modene = nvect->Mode;
  // Number of particles in event
  nework_.numne = nvect->Npart();

  for ( int i = 0; i < nework_.numne; i++ ) {
    nework_.ipne[i] = nvect->PartInfo(i)->fPID;
    nework_.pne[i][0] = (float)nvect->PartInfo(i)->fP.X() / 1000; // VC(NE)WORK in M(G)eV
    nework_.pne[i][1] = (float)nvect->PartInfo(i)->fP.Y() / 1000; // VC(NE)WORK in M(G)eV
    nework_.pne[i][2] = (float)nvect->PartInfo(i)->fP.Z() / 1000; // VC(NE)WORK in M(G)eV
  }

  // fsihist.h has documentaiton on fsihist common block

  // neutroot fills a dummy object for events with no FSI to prevent memory leak when reading the TTree, so check for it here
  // An event with FSI must have at least two vertices
  if ( (int)nvect->NfsiVert() == 1 ) {
    fsihist_.nvert = 0;
    fsihist_.nvcvert = 0;
    fsihist_.fsiprob = 1;
  // Real FSI event
  } else {
    fsihist_.nvert = (int)nvect->NfsiVert();
    for (int ivert = 0; ivert < fsihist_.nvert; ivert++) {
      fsihist_.iflgvert[ivert] = nvect->FsiVertInfo(ivert)->fVertID;
      fsihist_.posvert[ivert][0] = (float)nvect->FsiVertInfo(ivert)->fPos.X();
      fsihist_.posvert[ivert][1] = (float)nvect->FsiVertInfo(ivert)->fPos.Y();
      fsihist_.posvert[ivert][2] = (float)nvect->FsiVertInfo(ivert)->fPos.Z();
    }

    fsihist_.nvcvert = nvect->NfsiPart();
    for (int ip = 0; ip < fsihist_.nvcvert; ip++) {
      fsihist_.abspvert[ip] = (float)nvect->FsiPartInfo(ip)->fMomLab;
      fsihist_.abstpvert[ip] = (float)nvect->FsiPartInfo(ip)->fMomNuc;
      fsihist_.ipvert[ip] = nvect->FsiPartInfo(ip)->fPID;
      fsihist_.iverti[ip] = nvect->FsiPartInfo(ip)->fVertStart;
      fsihist_.ivertf[ip] = nvect->FsiPartInfo(ip)->fVertEnd;
      fsihist_.dirvert[ip][0] = (float)nvect->FsiPartInfo(ip)->fDir.X();
      fsihist_.dirvert[ip][1] = (float)nvect->FsiPartInfo(ip)->fDir.Y();
      fsihist_.dirvert[ip][2] = (float)nvect->FsiPartInfo(ip)->fDir.Z();
    }
    fsihist_.fsiprob = nvect->Fsiprob;
  }

  neutcrscom_.crsx = nvect->Crsx;
  neutcrscom_.crsy = nvect->Crsy;
  neutcrscom_.crsz = nvect->Crsz;
  neutcrscom_.crsphi = nvect->Crsphi;
  neutcrscom_.crsq2 = nvect->Crsq2;

  neuttarget_.numbndn = nvect->TargetA - nvect->TargetZ;
  neuttarget_.numbndp = nvect->TargetZ;
  neuttarget_.numfrep = nvect->TargetH;
  neuttarget_.numatom = nvect->TargetA;

  posinnuc_.ibound = nvect->Ibound;
  // put empty nucleon FSI history (since it is not saved in the NeutVect format)
  //Comment out as NEUT does not have the necessary proton FSI information yet
  //  nucleonfsihist_.nfnvert = 0;
  //  nucleonfsihist_.nfnstep = 0;
}

// ***************************
// Fill the NEUT MC params for each reconfigure of the engine from NeutVect object
// ___NEEDS TO BE KEPT IN-SYNC WITH neutclass/neutvect.h___
// ___WHICH IN TURNS GETS NVECT INFO FROM neutclass/neutfill.cc___
void NReWeight::FillNeutParams(NeutVect *nvect) {
// ***************************

  // First fill with the defaults from the NEUT necard.F, then over-write those with what is saved in the output file
//  necard_();

  // CCQE model
  nemdls_.mdlqe = nvect->QEModel;
  // Form factor CCQE model
  nemdls_.mdlqeaf = nvect->QEVForm;
  // 2p2h model
  nemdls_.mdl2p2h = nvect->MECModel;
  // Radiative correction
  neutradcorr_.iradcorr = nvect->RADcorr;

  // Single pion model
  nemdls_.mdlspi = nvect->SPIModel;
  // Rein-Sehgal single-pion ejection method
  nemdls_.mdlrs1piej = nvect->SPIRSEj;
  // Coherent model
  nemdls_.mdlcoh = nvect->COHModel;
  // DIS model
  nemdls_.mdldis = nvect->DISModel;
  neutdis_.nepdf = nvect->PDFModel;
  neutdis_.nebodek = nvect->DISBodekModel;
  neutdis_.nemult = nvect->DISMultiModel;
  // Diffractive model
  nemdls_.mdldif = nvect->DifModel;

  // Single pion form factors (Rein-Sehgal original or Graczyk & Sobczyk)
  nemdls_.iffspi = nvect->SPIForm;
  //0 = nvect->RESForm;

  // Single pion harmonic oscillator scaling (electron-scattering like or neutrino-scattering like)
  nemdls_.nrtypespi = nvect->SPINRType;
  //0 = nvect->RESNRType;

  // CCQE model parameters
  nemdls_.xmaqe = nvect->QEMA;
  nemdls_.xmvqe = nvect->QEMV;
  nemdls_.kapp = nvect->KAPPA;
  // Two/three component parameters
  nemdls_.axffalpha = nvect->QEFFTa;
  nemdls_.axffgamma = nvect->QEFFTg;
  nemdls_.axfftheta = nvect->QEFFTt;
  nemdls_.axffbeta = nvect->QEFFTb;
  // Z-expansion parameters
  nemdls_.axzexpq4 = nvect->QEFFZq4;
  nemdls_.axzexpnt = nvect->QEFFZnt;
  nemdls_.axzexpt0 = nvect->QEFFZt0;
  nemdls_.axzexptc = nvect->QEFFZtc;
  nemdls_.axzexpa0 = nvect->QEFFZ0;
  nemdls_.axzexpa1 = nvect->QEFFZ1;
  nemdls_.axzexpa2 = nvect->QEFFZ2;
  nemdls_.axzexpa3 = nvect->QEFFZ3;
  nemdls_.axzexpa4 = nvect->QEFFZ4;
  nemdls_.axzexpa5 = nvect->QEFFZ5;
  nemdls_.axzexpa6 = nvect->QEFFZ6;
  nemdls_.axzexpa7 = nvect->QEFFZ7;
  nemdls_.axzexpa8 = nvect->QEFFZ8;
  nemdls_.axzexpa9 = nvect->QEFFZ9;

  // Single pion model parameters
  nemdls_.xmaspi = nvect->SPIMA;
  nemdls_.xmvspi = nvect->SPIMV;
  nemdls_.xmares = nvect->RESMA;
  nemdls_.xmvres = nvect->RESMV;
  nemdls_.rca5ispi = nvect->SPICA5I;
  nemdls_.rbgsclspi = nvect->SPIBGScale;
  nemdls_.xmabkgm = nvect->SPIMaBKGM;

  // Coherent model parameters
  nemdls_.xmacoh = nvect->COHMA;
  nemdls_.rad0nu = nvect->COHR0;
  nemdls_.fa1coh = nvect->COHA1err;
  nemdls_.fb1coh = nvect->COHb1err;
  // Diffractive model parameters
  nemdls_.xmadif = nvect->DIFFMA;
  nemdls_.nucvoldif = nvect->DIFFVold;

  // NEUT versions
  neutversion_.corev = nvect->COREVer;
  neutversion_.nucev = nvect->NUCEVer;
  neutversion_.nuccv = nvect->NUCCVer;

  // NEUT card
  neutcard_.nefrmflg = nvect->FrmFlg;
  neutcard_.nepauflg = nvect->PauFlg;
  neutcard_.nenefo16 = nvect->NefO16;
  neutcard_.nemodflg = nvect->ModFlg;
  neutcard_.neselmod = nvect->SelMod;

  // Formation length
  nenupr_.iformlen = nvect->FormLen;

  // Pionless delta decay
  neutpiless_.ipilessdcy = nvect->IPilessDcy;
  neutpiless_.rpilessdcy = nvect->RPilessDcy;

  // Rescattering
  nucres_.nucrescat = nvect->NucScat;
  // Neutrino generating mode
  nucres_.xnucfact = nvect->NucFac;
  // Pion absorbtion
  neutpiabs_.neabspiemit = nvect->PiAbsEmit;

  // FSI scattering probabilities
  neffpr_.fefqe   = nvect->NuceffFactorPIQE;
  neffpr_.fefqeh  = nvect->NuceffFactorPIQEH;
  neffpr_.fefinel = nvect->NuceffFactorPIInel;
  neffpr_.fefabs  = nvect->NuceffFactorPIAbs;
  neffpr_.fefcx   = nvect->NuceffFactorPICX;
  neffpr_.fefcxh  = nvect->NuceffFactorPICXH;
  neffpr_.fefcoh  = nvect->NuceffFactorPICoh;
  neffpr_.fefqehf = nvect->NuceffFactorPIQEHKin;
  neffpr_.fefcohf = nvect->NuceffFactorPIQELKin;
  neffpr_.fefcxhf = nvect->NuceffFactorPICXKin;
  neffpr_.fefall  = nvect->NuceffFactorPIAll;
}

// ***************************
// Fill all the NEUT common blocks
void NReWeight::FillNeutCommons(NeutVect *nvect) {
// ***************************
  FillNeutEvent(nvect);
  FillNeutParams(nvect);
}


// ***************************
// Print all the parameters for the models, neutcore/neutmodel.h, neutcore/necard.h
void NReWeight::PrintNeutParams() {
  // ***************************

  // Input parameters
  std::cout << std::endl << "*** Variable Model Parameters ***" << std::endl;

  // CCQE
  std::cout << "  CCQE: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmaqe = " <<  nemdls_.xmaqe << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvqe = " <<  nemdls_.xmvqe << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.kapp  = " <<  nemdls_.kapp   << std::endl;
  std::cout << "------" << std::endl;

  // Coherent
  std::cout << "  Coherent: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmacoh = " <<         nemdls_.xmacoh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.rad0nu = " <<         nemdls_.rad0nu   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fa1coh = " <<         nemdls_.fa1coh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fb1coh = " <<         nemdls_.fb1coh   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.mdlcoh = " <<         nemdls_.mdlcoh   << std::endl;
  std::cout << "------" << std::endl;

  // 1pi mdl
  std::cout << "  Single pion: " << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmaspi = " <<      nemdls_.xmaspi   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvspi = " <<      nemdls_.xmvspi   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmares = " <<      nemdls_.xmares   << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.xmvres = " <<      nemdls_.xmvres   << std::endl;

  std::cout << std::left << std::setw(30) << "neut1pi_.xmanffres = " <<   neut1pi_.xmanffres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmvnffres = " <<   neut1pi_.xmvnffres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmarsres = " <<   neut1pi_.xmarsres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.xmvrsres = " <<   neut1pi_.xmvrsres << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.neiff = " <<   neut1pi_.neiff << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.nenrtype = " <<   neut1pi_.nenrtype << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.rneca5i = " <<   neut1pi_.rneca5i << std::endl;
  std::cout << std::left << std::setw(30) << "neut1pi_.rnebgscl = " <<   neut1pi_.rnebgscl << std::endl;
  std::cout << "------" << std::endl;

  // DIS/mpi
  std::cout << "  Mpi and DIS: " << std::endl;
  std::cout << std::left << std::setw(30) << "neutdis_.nepdf = " <<       neutdis_.nepdf   << std::endl;
  std::cout << std::left << std::setw(30) << "neutdis_.nebodek = " <<     neutdis_.nebodek  << std::endl;
  std::cout << "------" << std::endl;

  // Pion rescatttering
  std::cout << "  Pion rescattering:" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqe = " <<     neffpr_.fefqe   << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqeh = " <<    neffpr_.fefqeh  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefinel = " <<   neffpr_.fefinel << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefabs = " <<    neffpr_.fefabs  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcx = " <<     neffpr_.fefcx   << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcxh = " <<    neffpr_.fefcxh  << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefall = " <<    neffpr_.fefall  << std::endl;
  std::cout << "------" << std::endl;

  // Fixed parameters
  std::cout << "  Target materials:" << std::endl;

  std::cout << std::left << std::setw(30) << "neuttarget_.numbndn = " << neuttarget_.numbndn << std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numbndp = " <<   neuttarget_.numbndp << std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numfrep = " <<  neuttarget_.numfrep<< std::endl;
  std::cout << std::left << std::setw(30) << "neuttarget_.numatom = " <<  neuttarget_.numatom<< std::endl;

  std::cout << std::left << std::setw(30) << "nenupr_.pfsurf = " <<  nenupr_.pfsurf<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.pfmax = " <<  nenupr_.pfmax<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.vnuini = " << nenupr_.vnuini<< std::endl;
  std::cout << std::left << std::setw(30) << "nenupr_.vnufin = " << nenupr_.vnufin<< std::endl;
  std::cout << "------" << std::endl;


  std::cout << "  These are fixed (model) parameters [should = ()]" << std::endl;

  std::cout << std::left << std::setw(30) << "nemdls_.mdlqeaf    = " << nemdls_.mdlqeaf << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.mdlqe      = " << nemdls_.mdlqe << "(402)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.sccfv      = " << nemdls_.sccfv << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.sccfa      = " << nemdls_.sccfa << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "nemdls_.fpqe       = " << nemdls_.fpqe << "(1)" << std::endl;

  std::cout << std::left << std::setw(30) << "nenupr_.iformlen    = " << nenupr_.iformlen << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nefrmflg  = " << neutcard_.nefrmflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nepauflg  = " << neutcard_.nepauflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefo16  = " << neutcard_.nenefo16 << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nemodflg  = " << neutcard_.nemodflg << "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefmodl = " << neutcard_.nenefmodl << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefmodh = " << neutcard_.nenefmodh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nenefkinh = " << neutcard_.nenefkinh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutpiabs_.neabspiemit = " << neutpiabs_.neabspiemit << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neutcard_.nusim = " << neutcard_.nusim << "(1)" << std::endl;

  std::cout << std::left << std::setw(30) << "neutcoh_.necohepi  = " << neutcoh_.necohepi << "(0)" << std::endl;

  std::cout << std::left << std::setw(30) << "neffpr_.fefcoh  = " << neffpr_.fefcoh << "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefqehf = " << neffpr_.fefqehf<< "(1)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcxhf = " << neffpr_.fefcxhf<< "(0)" << std::endl;
  std::cout << std::left << std::setw(30) << "neffpr_.fefcohf = " << neffpr_.fefcohf<< "(0)" << std::endl;

  std::cout << std::endl;
}

// ***************************
// Print the NEUTCRS common block
void NReWeight::PrintNeutCrs() {
  // ***************************

  std::cout << "*** NEUTCRS Common Block ***" << std::endl;
  std::cout << "Crs[3] = " << neutcrscom_.crsx << " " << neutcrscom_.crsy << " " << neutcrscom_.crsz << " " << std::endl;
  std::cout << "Crsphi = " << neutcrscom_.crsphi << std::endl;
  std::cout << std::endl;
}

// ***************************
// Print the NEWORK common block (neutcore/nework.h)
void NReWeight::PrintNeutWork() {
  // ***************************

  std::cout << "*** NEWORK Common Block ***" << std::endl;
  std::cout << std::left << std::setw(10) << "Mode  = " << nework_.modene << std::endl;
  std::cout << std::left << std::setw(10) << "Numne = " << nework_.numne << std::endl;
  std::cout << std::left << std::setw(10) << "ipne" << std::setw(10) << "flag" << std::setw(15) << "origin part" << std::setw(10) << "alive" << std::setw(10) << "pne[3]" << std::endl;
  for (int ip=0; ip<nework_.numne; ip++) {
    std::cout << std::left << std::setw(10) << nework_.ipne[ip] << std::setw(10) << nework_.iflgne[ip] << std::setw(15) << nework_.iorgne[ip] << std::setw(10) << nework_.icrnne[ip];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << nework_.pne[ip][j] << ",";
    }
    std::cout << "}" << std::endl;
  }
  std::cout << std::endl;
}

// ***************************
// Print the FSI history
void NReWeight::PrintNeutFSIHist() {
  // ***************************

  std::cout << "*** FSI History Common Block *** " << std::endl;
  std::cout << std::left << std::setw(10) << "Nvert: " << fsihist_.nvert << std::endl;
  std::cout << std::left << std::setw(10) << "ivert" << std::setw(10) << "iflgvert" << std::setw(10) << "posvert[3]" << std::endl;

  for (int ivert = 0; ivert < fsihist_.nvert; ivert++) {
    std::cout << std::setw(10) << ivert << std::setw(10) << fsihist_.iflgvert[ivert];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << fsihist_.posvert[ivert][j] << ",";
    }
    std::cout << "}" << std::endl;
  }

  std::cout << std::left << std::setw(10) << "Nvcvert: " << fsihist_.nvcvert << std::endl;
  std::cout << std::left << std::setw(10) << "ip" << std::setw(10) << "ipvert" << std::setw(10) << "abspvert" << std::setw(10) << "iverti" << std::setw(10) << "ivertf" << std::setw(10) << "dirvert[3]" << std::endl;
  for (int ip=0; ip<fsihist_.nvcvert; ip++) {
    std::cout << std::setw(10) << ip << std::setw(10) << fsihist_.ipvert[ip] << std::setw(10) << fsihist_.abspvert[ip] << std::setw(10) << fsihist_.iverti[ip] << std::setw(10) << fsihist_.ivertf[ip];
    std::cout << "{";
    for (int j=0; j<3; j++) {
      std::cout << fsihist_.dirvert[ip][j] << ",";
    }
    std::cout << "}" << std::endl;
  }

  std::cout << "Fsiprob = " << fsihist_.fsiprob << std::endl;
  std::cout << "Ibound = " << posinnuc_.ibound << std::endl;

  std::cout << std::endl;
}

// ***************************
// Print the full NEUT common blocks
void NReWeight::PrintNeutAll() {
  // ***************************

  std::cout << "*** PRINTING FULL NEUT COMMON BLOCK ***" << std::endl;
  PrintNeutWork();
  PrintNeutFSIHist();
  PrintNeutParams();
  PrintNeutCrs();

}
