//____________________________________________________________________________
/*!

\class    neut::rew::NReWeight

\brief    Interface to the NEUT event reweighting engines

\author   Jim Dobson <J.Dobson07 \at imperial.ac.uk>
          Imperial College London

          Costas Andreopoulos <costas.andreopoulos \at stfc.ac.uk>
          STFC, Rutherford Appleton Laboratory

          Patrick de Perio <pdeperio \at physics.utoronto.ca>
	  University of Toronto

\created  Apr 5, 2011

\cpright  Copyright (c) 2003-2011, GENIE Neutrino MC Generator Collaboration
          For the full text of the license visit http://copyright.genie-mc.org
          or see $GENIE/LICENSE
*/
//____________________________________________________________________________

#ifndef _N_REWEIGHT_H_
#define _N_REWEIGHT_H_

// C++ std includes
#include <string>
#include <map>
#include <iostream>
#include <iomanip>
#include <vector>

// ROOT includes
#include <TMath.h>
#include <TString.h>

// Include all the NEUT reweight engines for interface ease
#include "NReWeightI.h"
#include "NReWeightCasc.h"
#include "NReWeightNuXSecCCQE.h"
#include "NReWeightNuXSecCCRES.h"
#include "NReWeightNuXSecCOH.h"
#include "NReWeightNuXSecDIS.h"
#include "NReWeightNuXSecNC.h"
#include "NReWeightNuXSecNCEL.h"
#include "NReWeightNuXSecNCRES.h"
#include "NReWeightNuXSecRES.h"
#include "NReWeightNuclPiless.h"
#include "NSyst.h"
#include "NSystSet.h"
#include "NSystUncertainty.h"

// The NEUT common blocks
#include "nefillverC.h"
#include "necardC.h"
#include "nrcardC.h"
#include "neutmodelC.h"
#include "neutparamsC.h"
#include "neworkC.h"
#include "fsihistC.h"
#include "neutcrsC.h"

// The NEUT class headers
#include "neutvect.h"
#include "neutpart.h"
#include "neutfsipart.h"
#include "neutfsivert.h"
#include "neutrootTreeSingleton.h"

namespace neut {

  namespace rew   {

    class NReWeight {

      public:
        NReWeight();
        ~NReWeight();

        void        AdoptWghtCalc (string name, NReWeightI* wcalc);   ///< add concrete weight calculator, transfers ownership
        NReWeightI* WghtCalc      (string name);                      ///< access a weight calculator by name
        NSystSet &  Systematics   (void);                             ///< set of enabled systematic params & values
        void        Reconfigure   (void);                             ///< reconfigure weight calculators with new params
        double      CalcWeight    ();                                 ///< calculate weight for input event
        double      CalcChisq     (void);                             ///< calculate penalty chisq for current values of tweaking dials
        void        Print         (void);                             ///< print

        double      GetWeightXsec () {return fWeightXsec;};           ///< weight from cross section variations only

        void       FillNeutEvent(NeutVect *nvect);
        void       FillNeutParams(NeutVect *nvect);
        void       FillNeutCommons(NeutVect *nvect);

        // Printers for NEUT
        void PrintNeutAll();
        void PrintNeutWork();
        void PrintNeutFSIHist();
        void PrintNeutParams();
        void PrintNeutCrs();

      private:

        void CleanUp (void);

        NSystSet                  fSystSet;   ///< set of enabled nuisance parameters
        map<string, NReWeightI *> fWghtCalc;  ///< concrete weight calculators

        double fWeightXsec;                   ///< store weight from cross section variations only

    };

  } // rew   namespace
} // neut namespace

#endif

