//____________________________________________________________________________
/*
 Copyright (c) 2003-2010, GENIE Neutrino MC Generator Collaboration
 For the full text of the license visit http://copyright.genie-mc.org
 or see $GENIE/LICENSE

 Authors: Costas Andreopoulos <costas.andreopoulos \at stfc.ac.uk>
          STFC, Rutherford Appleton Laboratory

          Jim Dobson <J.Dobson07 \at imperial.ac.uk>
          Imperial College London

	  Patrick de Perio <pdeperio \at physics.utoronto.ca>
	  University of Toronto

 For the class documentation see the corresponding header file.

 Important revisions after version 2.0.0 :
 @ Aug 01, 2009 - CA
   Was adapted from Jim's and Costas' T2K-specific GENIE reweighting code. 
   First included in v2.5.1.
 @ May 17, 2010 - CA
   Code extracted from NReWeightNuXSec and redeveloped in preparation for 
   the Summer 2010 T2K analyses.
 @ Oct 20, 2010 - CA
   Made static consts `kModeMaMv' and `kModeNormAndMaMvShape' public to
   aid external configuration.

   ... Missing lots of update information, don't rely on this log anymore!

 @ Summer 2017, Updated for Minoo's model reweighting
*/
//____________________________________________________________________________

#include <iostream>

#include <TMath.h>

#include "Controls.h"
#include "PDGCodes.h"
#include "NReWeightNuXSecCCRES.h"
#include "NSystSet.h"
#include "NSystUncertainty.h"
#include "NReWeightUtils.h"

//#define _N_REWEIGHT_CCRES_DEBUG_

using namespace neut;
using namespace neut::rew;

using std::cout;
using std::endl;

const int NReWeightNuXSecCCRES::kModeMaMv;
const int NReWeightNuXSecCCRES::kModeNormAndMaShape;

//_______________________________________________________________________________________
NReWeightNuXSecCCRES::NReWeightNuXSecCCRES() 
{
  this->Init();
}
//_______________________________________________________________________________________
NReWeightNuXSecCCRES::~NReWeightNuXSecCCRES()
{
}
//_______________________________________________________________________________________
void NReWeightNuXSecCCRES::Init(void)
{
  fortFns = NFortFns::Instance();

  neutTotCrs = NTotCrs::Instance();

  // Set the mode to be MaMv reweight
  // SUPER IMPORTANT BECAUSE THIS DECIDES ALL THE REWEIGHTING
  // C. Wret no idea why we still have this option, relic?
  this->SetMode(kModeMaMv);

  this->RewNue    (true);
  this->RewNuebar (true);
  this->RewNumu   (true);
  this->RewNumubar(true);

  fNormTwkDial = 0.;
  fNormDef     = 1.;
  fNormCurr    = fNormDef;
  /*
  fMaTwkDial   = 0.; 
  fMaDef       = fortFns->XMACCRESdef;
  fMaCurr      = fMaDef;
  fMvTwkDial   = 0.; 
  fMvDef       = fortFns->XMVCCRESdef;
  fMvCurr      = fMvDef;
  */
  fMaNFFTwkDial   = 0.; 
  fMaNFFDef       = 0.; 
  fMaNFFCurr      = fMaNFFDef;
  fMvNFFTwkDial   = 0.; 
  fMvNFFDef       = 0.; 
  fMvNFFCurr      = fMvNFFDef;
  fMaRSTwkDial   = 0.; 
  fMaRSDef       = 0.; 
  fMaRSCurr      = fMaRSDef;
  fMvRSTwkDial   = 0.; 
  fMvRSDef       = 0.; 
  fMvRSCurr      = fMvRSDef;
  fCA5TwkDial   = 0.; 
  fCA5Def       = 0.; 
  fCA5Curr      = fCA5Def;
  fBgSclTwkDial   = 0.; 
  fBgSclDef       = 0.; 
  fBgSclCurr      = fBgSclDef;
  fIFFTwkDial   = 0; 
  fIFFDef       = 0.; 
  fIFFCurr      = fIFFDef;
  fNRTypeTwkDial   = 0; 
  fNRTypeDef       = 0.; 
  fNRTypeCurr      = fNRTypeDef;


  fMaBKGMTwkDial   = 0; 
  fMaBKGMDef       = 0.; 
  fMaBKGMCurr      = fMaBKGMDef;

  fSPPModelTwkDial   = 0; 
  fSPPModelDef     = 0.; 
  fSPPModelCurr    = fSPPModelDef;

  fRSPiEjTwkDial   = 0; 
  fRSPiEjDef       = 0.; 
  fRSPiEjCurr      = fRSPiEjDef;

}

//_______________________________________________________________________________________
// Check that the systematic is handled by the CCRES reweighting engine
bool NReWeightNuXSecCCRES::IsHandled(NSyst_t syst)
{
  bool handle;

  switch(syst) {

    case ( kXSecTwkDial_MaCCRESshape ) :
    case ( kXSecTwkDial_NormCCRES    ) :
    case ( kXSecTwkDial_MaNFFCCRES ) :
    case ( kXSecTwkDial_MvNFFCCRES ) :
    case ( kXSecTwkDial_MaRSCCRES ) :
    case ( kXSecTwkDial_MvRSCCRES ) :
    case ( kXSecTwkDial_CA5CCRES ) :
    case ( kXSecTwkDial_BgSclCCRES ) :
    case ( kXSecTwkDial_FFCCRES ) :
    case ( kXSecTwkDial_TypeCCRES ) :
    case ( kXSecTwkDial_MaBKGM ) :
    case ( kXSecTwkDial_SPPModel ) :
    case ( kXSecTwkDial_RSPiEj ) :
      handle = true;
      break;

    default:
      handle = false;
      break;
  }

  return handle;
}
//_______________________________________________________________________________________
void NReWeightNuXSecCCRES::SetSystematic(NSyst_t syst, double twk_dial)
{
  // Check if this reweight engine handles the systematic
  if(!this->IsHandled(syst)) return;

  // Set the systematic to their values when handled
  switch(syst) {

    // Normalisation of CCRES
    case ( kXSecTwkDial_NormCCRES ) :
      fNormTwkDial = twk_dial;
      break;

      // MARES for Graczyk Sobczyk
    case ( kXSecTwkDial_MaNFFCCRES ) :
      fMaNFFTwkDial = twk_dial;
      break;

      // MVRES for Graczyk Sobczyk
    case ( kXSecTwkDial_MvNFFCCRES ) :
      fMvNFFTwkDial = twk_dial;
      break;

      // MARES for Rein-Sehgal
    case ( kXSecTwkDial_MaRSCCRES ) :
      fMaRSTwkDial = twk_dial;
      break;

      // MVRES for Rein-Sehgal
    case ( kXSecTwkDial_MvRSCCRES ) :
      fMvRSTwkDial = twk_dial;
      break;

      // CA5(0) for Graczyk Sobczyk
    case ( kXSecTwkDial_CA5CCRES ) :
      fCA5TwkDial = twk_dial;
      break;

      // I12 background
    case ( kXSecTwkDial_BgSclCCRES ) :
      fBgSclTwkDial = twk_dial;
      break;

      // Graczyk Sobczyk form factors or Rein-Sehgal form factors
    case ( kXSecTwkDial_FFCCRES ) :
      fIFFTwkDial = twk_dial;
      break;

      // How to scale form factor with quark oscillator level (electron-scattering like or neutrino-scattering like)
    case ( kXSecTwkDial_TypeCCRES ) :
      fNRTypeTwkDial = twk_dial;
      break;

      // Minoo's MA for the background terms
    case ( kXSecTwkDial_MaBKGM ) :
      fMaBKGMTwkDial = twk_dial;
      break;

      // Switch between Rein-Sehgal (with or without Graczyk & Sobczyk form factors) and Minoo's model
    case ( kXSecTwkDial_SPPModel ) :
      fSPPModelTwkDial = twk_dial;
      break;

    // Switch between different single pion ejection methods in the Rein-Sehgal model
    case ( kXSecTwkDial_RSPiEj ) :
      fRSPiEjTwkDial = twk_dial;
      break;

    default:
      break;
  }
}
//_______________________________________________________________________________________
// Reset the dials to their original values
void NReWeightNuXSecCCRES::Reset(void)
{
  fNormTwkDial = 0.;
  fNormCurr    = fNormDef;
  /*
     fMaTwkDial   = 0.; 
     fMaCurr      = fMaDef;
     fMvTwkDial   = 0.; 
     fMvCurr      = fMvDef;
 */
  fMaNFFTwkDial   = 0.; 
  fMaNFFCurr      = fMaNFFDef;

  fMvNFFTwkDial   = 0.; 
  fMvNFFCurr      = fMvNFFDef;

  fMaRSTwkDial   = 0.; 
  fMaRSCurr      = fMaRSDef;

  fMvRSTwkDial   = 0.; 
  fMvRSCurr      = fMvRSDef;

  fCA5TwkDial   = 0.; 
  fCA5Curr      = fCA5Def;

  fBgSclTwkDial   = 0.; 
  fBgSclCurr      = fBgSclDef;

  fIFFTwkDial   = 0; 
  fIFFCurr      = fIFFDef;

  fNRTypeTwkDial   = 0; 
  fNRTypeCurr      = fNRTypeDef;

  // Minoo background axial mass
  fMaBKGMTwkDial = 0.0;
  fMaBKGMCurr = fMaBKGMDef;

  this->Reconfigure();
}
//_______________________________________________________________________________________
void NReWeightNuXSecCCRES::Reconfigure(void)
{
  NSystUncertainty * fracerr = NSystUncertainty::Instance();

  // This is what is called in NEUT
  // fMode is set to kModeMaMv in the constructor
  if(fMode==kModeMaMv) {   
    double fracerr_ma = fracerr->OneSigmaErr(kXSecTwkDial_MaCCRES);
    double fracerr_mv = fracerr->OneSigmaErr(kXSecTwkDial_MvCCRES);
    fMaCurr = (1. + fMaTwkDial * fracerr_ma);
    fMvCurr = (1. + fMvTwkDial * fracerr_mv);

    // This is seemingly never called
  } else if (fMode==kModeNormAndMaShape) { 
    double fracerr_mash = fracerr->OneSigmaErr(kXSecTwkDial_MaCCRESshape);
    fMaCurr   = fMaDef   * (1. + fMaTwkDial   * fracerr_mash);    
    //double fracerr_mvsh = fracerr->OneSigmaErr(kXSecTwkDial_MvCCRESshape);
    //fMvCurr   = fMvDef   * (1. + fMvTwkDial   * fracerr_mvsh);
  }

  // *****************************************************
  // Set the new normalisation
  double fracerr_norm = fracerr->OneSigmaErr(kXSecTwkDial_NormCCRES);
  fNormCurr = fNormDef * (1. + fNormTwkDial * fracerr_norm);     
  // *****************************************************

  // *****************************************************
  // Set the new MA and MV for the original Rein-Sehgal form factors
  double fracerr_rsma = fracerr->OneSigmaErr(kXSecTwkDial_MaRSCCRES);
  fMaRSCurr = (1. + fMaRSTwkDial * fracerr_rsma);
  double fracerr_rsmv = fracerr->OneSigmaErr(kXSecTwkDial_MvRSCCRES);
  fMvRSCurr = (1. + fMvRSTwkDial * fracerr_rsmv);
  // *****************************************************

  // *****************************************************
  // Graczyk Sobczyk form factor model variables
  // ____ Minoo (Monireh Kabirnezhad) model also uses this MA, MV and CA5(0) ____
  // Set the new MA and MV for the Graczyk & Sobczyk form factors
  double fracerr_nffma = fracerr->OneSigmaErr(kXSecTwkDial_MaNFFCCRES);
  fMaNFFCurr = (1. + fMaNFFTwkDial * fracerr_nffma);
  double fracerr_nffmv = fracerr->OneSigmaErr(kXSecTwkDial_MvNFFCCRES);
  fMvNFFCurr = (1. + fMvNFFTwkDial * fracerr_nffmv);

  // Set the new CA5 and I12 for Graczyk & Sobczyk form factors
  double fracerr_ca5 = fracerr->OneSigmaErr(kXSecTwkDial_CA5CCRES);
  fCA5Curr = (1. + fCA5TwkDial * fracerr_ca5);
  double fracerr_bgscl = fracerr->OneSigmaErr(kXSecTwkDial_BgSclCCRES);
  fBgSclCurr = (1. + fBgSclTwkDial * fracerr_bgscl);
  // End Graczyk Sobczyk form factor model variables
  // *****************************************************

  // *****************************************************
  // Minoo model specific: MA of background term
  double fracerr_mabkgm = fracerr->OneSigmaErr(kXSecTwkDial_MaBKGM);
  fMaBKGMCurr = (1. + fMaBKGMTwkDial * fracerr_mabkgm);
  // *****************************************************

  // *****************************************************
  // Model switch parameters ___ NOT CONTINUOUS! ____
  // Set the new value for the form factor choice (Graczyk & Sobczyk vs Rein-Sehgal original)
  // Only applies to Rein-Sehgal and Graczyk Sobczyk form factors
  double fracerr_iff = fracerr->OneSigmaErr(kXSecTwkDial_FFCCRES);
  fIFFCurr = int(fracerr_iff * fIFFTwkDial);

  // Set the new value for the quark oscillator level scheme (both Graczyk & Sobczyk and Rein-Sehgal propose different behaviour for the form factors when using electron scattering and neutrino scattering)
  double fracerr_type = fracerr->OneSigmaErr(kXSecTwkDial_TypeCCRES); 
  fNRTypeCurr = int(fracerr_type * fNRTypeTwkDial);

  // Reweight between single pion production models in NEUT. Definitely ___NOT___ trustworthy or should ever be relied on
  double fracerr_sppm = fracerr->OneSigmaErr(kXSecTwkDial_SPPModel);
  fSPPModelCurr = int(fracerr_sppm * fSPPModelTwkDial);

  // Reweight between Rein-Sehgal pion ejection models in NEUT. Definitely ___NOT___ trustworthy or should ever be relied on
  double fracerr_rspiej = fracerr->OneSigmaErr(kXSecTwkDial_RSPiEj);
  fRSPiEjCurr = int(fracerr_rspiej * fRSPiEjTwkDial);
  // *****************************************************
}
//_______________________________________________________________________________________
double NReWeightNuXSecCCRES::CalcWeight() 
{
  bool is_res = modeDefn.isCCRES(nework_.modene);

  if (!is_res) return 1.;

  double wght = 1.0;

  // Get the neutrino type and check we want to reweight it
  int nupdg = nework_.ipne[0];
  if(nupdg==kPdgNuMu     && !fRewNumu   ) return 1.;
  if(nupdg==kPdgAntiNuMu && !fRewNumubar) return 1.;
  if(nupdg==kPdgNuE      && !fRewNue    ) return 1.;
  if(nupdg==kPdgAntiNuE  && !fRewNuebar ) return 1.;

  // Calculate the weight from model variations (MaMv) and normalisation change
  wght *= this->CalcWeightMaMv()*this->CalcWeightNorm();

  return wght;
}
//_______________________________________________________________________________________
// _____ THIS IS SERIOUSLY DEPRECATED AND NEEDS UPDATING  IF TO BE USED ______
double NReWeightNuXSecCCRES::CalcChisq()
{
  double chisq = 0.;
  chisq += TMath::Power(fMaTwkDial, 2.);
  chisq += TMath::Power(fMvTwkDial, 2.);
  chisq += TMath::Power(fNormTwkDial, 2.);

  return chisq;
}
//_______________________________________________________________________________________
// Apply normalisation weight
double NReWeightNuXSecCCRES::CalcWeightNorm() 
{
  bool tweaked = (TMath::Abs(fNormTwkDial) > controls::kASmallNum);
  if(!tweaked) return 1.0;

  double wght = fNormCurr;
  return wght;
}
//_______________________________________________________________________________________
double NReWeightNuXSecCCRES::CalcWeightMaMv() 
{

  // Check if we've changed parameters
  bool tweaked = 
    (TMath::Abs(fMaNFFTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fMvNFFTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fMaRSTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fMvRSTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fCA5TwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fBgSclTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fIFFTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fNRTypeTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fMaTwkDial) > controls::kASmallNum) ||
    (TMath::Abs(fMvTwkDial) > controls::kASmallNum);
  if (!tweaked) return 1.0;

  fortFns->SetDefaults();
  fortFns->Reconfigure();

#ifdef _N_REWEIGHT_CCRES_DEBUG_
  fortFns->print_allevent();
  fortFns->print_allparams();
#endif

  // Calculate the old cross-section (default cross-section)
  float old_xsec     = fortFns->evdifcrs();

  if (old_xsec == 0) {
    cout << "NReWeightNuXSecCCRES::CalcWeightMa() Warning: old_xsec==0, setting weight to 1" << endl;
    return 1.0;
  }

  // Read the current model parameter variations
  // xmaspi is not directly changed anymore: instead changed xmanffres or xmasrsres. xmaspi is then set to these variables inside of NEUT
  fMaDef = nemdls_.xmaspi;
  fMvDef = nemdls_.xmvspi;

  fMaNFFDef = neut1pi_.xmanffres;
  fMvNFFDef = neut1pi_.xmvnffres;

  fMaRSDef = neut1pi_.xmarsres;
  fMvRSDef = neut1pi_.xmvrsres;

  fIFFDef = neut1pi_.neiff;
  fNRTypeDef = neut1pi_.nenrtype;

  fCA5Def = neut1pi_.rneca5i;
  fBgSclDef = neut1pi_.rnebgscl;

  // Slight inconsistency here: MA background lives in the neut model common block, not the neut1pi common block
  // The neut1pi common block gets written to neutmdls every iteration so I chose to just to read directly from neutmdls instead
  fMaBKGMDef = nemdls_.xmabkgm;
  fSPPModelDef = nemdls_.mdlspi;
  fRSPiEjDef = nemdls_.mdlrs1piej;

  // Make sure nothing is negative
  neut1pi_.xmanffres  = TMath::Max(0., fMaNFFCurr * fMaNFFDef);
  neut1pi_.xmvnffres  = TMath::Max(0., fMvNFFCurr * fMvNFFDef);
  neut1pi_.xmarsres   = TMath::Max(0., fMaRSCurr * fMaRSDef);
  neut1pi_.xmvrsres   = TMath::Max(0., fMvRSCurr * fMvRSDef);
  neut1pi_.neiff      = TMath::Max(0, fIFFCurr + fIFFDef);
  neut1pi_.nenrtype   = TMath::Max(0, fNRTypeCurr + fNRTypeDef);
  neut1pi_.rneca5i    = TMath::Max(0., fCA5Curr * fCA5Def);
  neut1pi_.rnebgscl   = TMath::Max(0., fBgSclCurr * fBgSclDef);

  nemdls_.xmabkgm     = TMath::Max(0., fMaBKGMCurr * fMaBKGMDef);
  nemdls_.mdlspi      = TMath::Max(0, fSPPModelCurr + fSPPModelDef);
  nemdls_.mdlrs1piej  = TMath::Max(0, fRSPiEjCurr + fRSPiEjDef);

  fortFns->Reconfigure();

#ifdef _N_REWEIGHT_CCRES_DEBUG_
  fortFns->print_allevent();
  fortFns->print_allparams();
#endif

  float new_xsec   = fortFns->evdifcrs();
  float new_weight = (new_xsec/old_xsec);

#ifdef _N_REWEIGHT_CCRES_DEBUG_
  cout << "differential cross section (old) = " << old_xsec << endl;
  cout << "differential cross section (new) = " << new_xsec << endl;
#endif

  // ______ THIS IS NEVER USED IN NEUT BECAUSE fMode IS ___ALWAYS___ SET TO kModeMaMv ______
  // Normalize out change in total cross section from MA variations for MA_shape variation
  if(fMode==kModeNormAndMaShape) {

    // Get neutrino type
    int inu=-1;
    if (nework_.ipne[0] == 14) inu = neutTotCrs->numu;
    else if (nework_.ipne[0] == -14) inu = neutTotCrs->numub;
    else if (nework_.ipne[0] == 12) inu = neutTotCrs->nue;
    else if (nework_.ipne[0] == -12) inu = neutTotCrs->nueb;
    else {
      cout << "NReWeightNuXSecCCRES::CalcWeightMa() Error: Cannot MA-shape reweight this neutrino type = " 
        << nework_.ipne[0] << endl;
      exit (-1);
    }

    // Calculate neutrino energy
    float Enu = 0;  // GeV
    for (int i=0; i<3; i++)
      Enu += nework_.pne[0][i]*nework_.pne[0][i];
    Enu = sqrt(Enu);

    // Get reaction mode
    int imode=-1;
    if      (abs(nework_.modene)==11) imode = neutTotCrs->neutmode11;
    else if (abs(nework_.modene)==12) imode = neutTotCrs->neutmode12;
    else if (abs(nework_.modene)==13) imode = neutTotCrs->neutmode13;
    else if (abs(nework_.modene)==31) imode = neutTotCrs->neutmode31;
    else if (abs(nework_.modene)==32) imode = neutTotCrs->neutmode32;
    else if (abs(nework_.modene)==33) imode = neutTotCrs->neutmode33;
    else if (abs(nework_.modene)==34) imode = neutTotCrs->neutmode34;
    else {
      //  cout << "NReWeightNuXSecCCRES::CalcWeightMa() Warning: Cannot reweight MaShape for mode = " 
      //       << nework_.modene << ", setting weight = 1" << endl;
      return 1;
    }

    float old_tot_xsec = neutTotCrs->resspi_crs[inu][imode]->Interpolate(Enu, fMaDef);
    float new_tot_xsec = neutTotCrs->resspi_crs[inu][imode]->Interpolate(Enu, fMaCurr);

    if (new_tot_xsec==0) {
      cout << "NReWeightNuXSecCCRES::CalcWeightMa() Warning: new_tot_xsec==0, setting weight to 1" << endl;
      return 1;
    }

#ifdef _N_REWEIGHT_CCRES_DEBUG_
    cout << "total cross section (old) = " << old_tot_xsec << endl;
    cout << "total cross section (new) = " << new_tot_xsec << endl;
#endif 

    new_weight *= old_tot_xsec / new_tot_xsec ;
  }

  if (isinf(new_weight) || isnan(new_weight)) {
    cout << "NReWeightNuXSecCCRES::CalcWeightMa() Warning: new_weight is infinite, setting to 1" << endl;
    new_weight = 1.0;
  }

#ifdef _N_REWEIGHT_CCRES_DEBUG_
  cout << "new weight = " << new_weight << endl;
#endif

  return new_weight;
}

//_______________________________________________________________________________________
double NReWeightNuXSecCCRES::CalcWeightMaMvShape() 
{
  return 1.0;

  /* 
  //GENIE Implementation below
     bool tweaked = 
     (TMath::Abs(fMaTwkDial) > controls::kASmallNum) ||
     (TMath::Abs(fMvTwkDial) > controls::kASmallNum);
     if(!tweaked) return 1.0;

     Interaction * interaction = event.Summary();

     interaction->KinePtr()->UseSelectedKinematics();

     double old_xsec   = event.DiffXSec();
     double old_weight = event.Weight();
     double new_xsec   = fXSecModel->XSec(interaction, kPSWQ2fE);
     double new_weight = old_weight * (new_xsec/old_xsec);

  //LOG("ReW", pDEBUG) << "differential cross section (old) = " << old_xsec;
  //LOG("ReW", pDEBUG) << "differential cross section (new) = " << new_xsec;
  //LOG("ReW", pDEBUG) << "event generation weight = " << old_weight;
  //LOG("ReW", pDEBUG) << "new weight = " << new_weight;

  //double old_integrated_xsec = event.XSec();
  double old_integrated_xsec = fXSecModelDef -> Integral(interaction);
  double twk_integrated_xsec = fXSecModel    -> Integral(interaction);
  assert(twk_integrated_xsec > 0);
  new_weight *= (old_integrated_xsec/twk_integrated_xsec);

  //LOG("ReW", pDEBUG) << "integrated cross section (old) = " << old_integrated_xsec;
  //LOG("ReW", pDEBUG) << "integrated cross section (twk) = " << twk_integrated_xsec;
  //LOG("ReW", pDEBUG) << "new weight (normalized to const integral) = " << new_weight;

  interaction->KinePtr()->ClearRunningValues();

#ifdef _N_REWEIGHT_CCRES_DEBUG_
double E  = interaction->InitState().ProbeE(kRfHitNucRest);
double Q2 = interaction->Kine().Q2(true);
double W  = interaction->Kine().W(true);
fTestNtp->Fill(E,Q2,W,new_weight);
#endif

return new_weight;
*/
}
